<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 44f79311fc0c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-03-23 19:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_document_other extends we_document_binary{

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::APPLICATION;

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		if(isWE()){
			$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_PREVIEW;
		}
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				return new we_editor_other_preview($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	/* gets the HTML for including in HTML-Docs */

	function getHtml(bool $dyn = false): string{
		$data = $this->getElementS('data');
		$this->html = ($this->ID || ($data && !is_dir($data) && is_readable($data)) ?
				'<p><b>Datei</b>: ' . $this->getText() . '</p>' :
				g_l('global', '[no_file_uploaded]'));

		return $this->html;
	}

	protected function formExtension(): string{
		return we_html_tools::formElementTable(g_l('weClass', '[extension]'), we_html_tools::textInput('we_' . $this->Name . '_Extension', $this->Extension, 0, ['oninput' => "_EditorFrame.setEditorIsHot(true);",
							'style' => 'width:92px;']));
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		return parent::we_save($resave, $skipHook);
	}

	/**
	 * create instance of weMetaData to access metadata functionality:
	 */
	protected function getMetaDataReader(bool $force = false): we_metadata_metaData{
		throw new we_exception_base('not available');
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []): bool{
		if(!($this->IsSearchable && $this->Publish_Date)){
			$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID));
			return true;
		}

		$content = '';
		foreach($this->nextElement('txt') as $k){
			assert(is_string($k));
			$v = trim($this->getElementS($k));
			if($v && $k != 'data'/* filename */){
				$content .= ' ' . $v;
			}
		}

		switch($this->Extension){
			case '.doc':
			case '.xls':
			case '.pps':
			case '.ppt':
				break;
			case '.pdf':
			case '.rtf':
				$content .= ' ' . $this->getDocumentData('', 1000000);
				break;
			case '.odt':
			case '.ods':
			case '.odf':
			case '.odp':
			case '.odg':
			case '.ott':
			case '.ots':
			case '.otf':
			case '.otp':
			case '.otg':
				$name = $this->getElementS('data');
				if(class_exists('ZipArchive', true) && file_exists($name)){
					$zip = new ZipArchive;
					if($zip->open($name) === true){
						$content .= ' ' . strip_tags(preg_replace(['|</text[^>]*>|', '|<text[^/>]*/>|'], ' ', str_replace(['&#x0d;', '&#x0a;'], ' ', $zip->getFromName('content.xml') ?: '')));
						$zip->close();
						break;
					}
				}
				break;
		}

		$text = self::formatIndexText($content);

		$this->DB_WE->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID=' . intval($this->ID) . ($text ? ' AND WorkspaceID!=' . intval($this->ParentID) : ''));
		if(!$text){
			return true;
		}

		return $this->DB_WE->query('REPLACE INTO ' . INDEX_TABLE . ' SET ' . we_database_base::arraySetter([
							'ID' => $this->ID,
							'Text' => $text,
							'WorkspaceID' => $this->ParentID,
							'Category' => implode(',', array_filter(array_unique($this->Category))) ?: null,
							'Title' => self::formatIndexText($this->getElementS(self::TITLE_FIELD), 255) ?: null,
							'Description' => self::formatIndexText($this->getElementS(self::DESCRIPTION_FIELD)) ?: null,
							'Keywords' => self::formatIndexText($this->getElementS(self::KEYWORD_FIELD)) ?: null,
		]));
	}

	public function setMetaDataFromFile(string $file): void{
		if($this->Extension === '.pdf' && file_exists($file)){
			t_e('wrong documenttype passed!');
		}
	}

	static function checkAndPrepare(string $formname, we_contents_root $doc): void{
		$reqName = 'we_ui_' . $formname;
		// check to see if there is an image to create or to change
		if(empty($_FILES[$reqName]) || empty($_FILES[$reqName]['name']) || !is_array($_FILES[$reqName]['name'])){
			return;
		}
		foreach($_FILES[$reqName]['name'] as $binaryName => $filename){
			'@phan-var string $binaryName';
			'@phan-var string $filename';
			$binaryDataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_BINARY . $binaryName);

			if(!$binaryDataId || !isset($_SESSION[$binaryDataId])){
				continue;
			}

			if(we_base_request::bool(we_fileupload::USERINPUT_TYPE_DEL . $binaryName)){
				$_SESSION[$binaryDataId]['doDelete'] = true;
				continue;
			}
			$_SESSION[$binaryDataId]['doDelete'] = false;
			if($filename){
				// file is selected, check to see if it is an image
				if(!in_array(we_base_ContentTypes::getFromFilename($filename), [we_base_ContentTypes::APPLICATION, we_base_ContentTypes::PDF], true)){
					continue;
				}
				$binaryId = $doc->getElementI($binaryName);

				// move document from upload location to tmp dir
				$_SESSION[$binaryDataId]['serverPath'] = $serverPath = TEMP_PATH . we_base_file::getUniqueId();
				we_base_file::insertIntoCleanUp($serverPath, 3600);
				move_uploaded_file((string) $_FILES[$reqName]['tmp_name'][$binaryName], $serverPath);
				$unique = we_base_file::getUniqueId();
				$tmp_Filename = $binaryName . '_' . $unique . '_' . preg_replace('/[^A-Za-z0-9._-]/', '', (string) $_FILES[$reqName]['name'][$binaryName]);
				if($binaryId){
					$_SESSION[$binaryDataId]['id'] = $binaryId;
				}

				$_SESSION[$binaryDataId]['fileName'] = preg_replace('#^(.+)\..+$#', '${1}', $tmp_Filename);
				$_SESSION[$binaryDataId]['extension'] = (strpos($tmp_Filename, '.') > 0) ? preg_replace('#^.+(\..+)$#', '${1}', $tmp_Filename) : '';
				$_SESSION[$binaryDataId]['text'] = $_SESSION[$binaryDataId]['fileName'] . $_SESSION[$binaryDataId]['extension'];
				$_SESSION[$binaryDataId]['type'] = $_FILES[$reqName]['type'][$binaryName];
				$_SESSION[$binaryDataId]['size'] = $_FILES[$reqName]['size'][$binaryName];
				$_SESSION[$binaryDataId]['unique'] = $unique;
			}
		}
	}

}
