<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f2413f100800 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-01-29 10:30 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_ExportCSV extends we_exim_Export{
	/** @var string */
	protected $exportType = we_exim_ExIm::TYPE_CSV;

	const LINEEND_WINDOWS = 'windows';
	const LINEEND_MAC = 'mac';
	const LINEEND_UNIX = 'unix';

	/** @var string[] */
	protected $permittedContentTypes = [
		we_base_ContentTypes::OBJECT_FILE,
	];

	/** @var array<string,string> */
	private const delimiters = [
		'semicolon' => ';',
		'comma' => ',',
		'colon' => ':',
		'tab' => "\t",
		'space' => ' '
	];

	/** @var array<string,string> */
	private const encloses = [
		'singlequote' => '\'',
		'doublequote' => '"'
	];

	/** @var array<string,string> */
	private const lineends = [
		self::LINEEND_UNIX => "\n",
		self::LINEEND_MAC => "\r",
		self::LINEEND_WINDOWS => "\r\n"
	];

	function __construct(){
		parent::__construct();
		$this->export_binary = false;
		$this->permittedContentTypes[] = we_base_ContentTypes::WEDOCUMENT;
	}

	public static function getCSVDelimeters(): array{
		return [
			';' => g_l('modules_customer', '[semicolon]'),
			',' => g_l('modules_customer', '[comma]'),
			':' => g_l('modules_customer', '[colon]'),
			'\t' => g_l('modules_customer', '[tab]'),
			' ' => g_l('modules_customer', '[space]')];
	}

	public static function getCSVQuotes(): array{
		return [
			'"' => g_l('modules_customer', '[double_quote]'),
			"'" => g_l('modules_customer', '[single_quote]')];
	}

	public static function getCSVLineEnds(): array{
		return [
			self::LINEEND_WINDOWS => g_l('modules_customer', '[windows]'),
			self::LINEEND_UNIX => g_l('modules_customer', '[unix]'),
			self::LINEEND_MAC => g_l('modules_customer', '[mac]')
		];
	}

	protected function fileCreate(): bool{
		if(!parent::fileCreate()){
			return false;
		}
		$header = '';
		we_base_file::save($this->exportProperties['file'], $header);
		return true;
	}

	protected function writeExportItem(we_base_mainmodel $doc, array $attribute = []): void{
		if($doc instanceof we_backup_binary){
			return;
		}
		if($doc instanceof we_contents_root && in_array($doc->ContentType, $this->permittedContentTypes, true)){
			switch($doc->Table){
				case OBJECT_FILES_TABLE:
					assert($doc instanceof we_contents_objectFile);
					$this->object2nonWE($doc);
					break;
				case FILE_TABLE:
					$this->document2nonWE($doc);
					break;
			}
			assert(is_resource($this->fh));
			fwrite($this->fh, self::lineends[$this->options['csv_lineend']] ?? self::lineends['windows']); //@phan-suppress-current-line PhanCoalescingNeverNull
		}
	}

	protected function formatOutput(string $content, string $tagname = '', string $format = we_exim_ExIm::TYPE_XML, int $tabs = 2, bool $cdata = false, bool $fix_content = false): string{
		$delimiter = self::delimiters[$this->options['csv_delimiter']];

		return ($content ? self::correctCSV($content) . $delimiter : $delimiter);
	}

	protected static function checkCompatibility(string $content, string $csv_delimiter = ",", string $csv_enclose = "'", string $type = "escape"): bool{
		$check = [];
		switch($type){
			case 'escape':
				$check = ["\\"];
				break;
			case 'enclose':
				$check = [$csv_enclose];
				break;
			case 'delimiter':
				$check = [$csv_delimiter];
				break;
			case 'lineend':
				$check = ["\r\n", "\n", "\r"];
				break;
		}

		foreach($check as $cur){
			if(str_contains($content, $cur)){
				return true;
			}
		}

		return false;
	}

	protected function correctCSV(string $content): string{
		$encloser_corrected = false;
		$delimiter_corrected = false;
		$lineend_corrected = false;

		$encloser = self::encloses[$this->options['csv_enclose']] ?? '\''; //@phan-suppress-current-line PhanCoalescingNeverNull
		$delimiter = self::delimiters[$this->options['csv_delimiter']] ?? ','; //@phan-suppress-current-line PhanCoalescingNeverNull
		$lineend = $this->options['csv_lineend'] ?: self::LINEEND_WINDOWS;

		// Escape
		$corrected_content = (self::checkCompatibility($content, $delimiter, $encloser, 'escape') ?
			self::correctEscape($content) : $content);

		// Enclose
		if(self::checkCompatibility($corrected_content, $delimiter, $encloser, 'enclose')){
			$encloser_corrected = true;

			$corrected_content = self::correctEnclose($corrected_content, $encloser);
		} else {
			$corrected_content = $content;
		}

		// Delimiter
		if(self::checkCompatibility($corrected_content, $delimiter, $encloser, 'delimiter')){
			$delimiter_corrected = true;
		}

		// Lineend
		if(self::checkCompatibility($corrected_content, $delimiter, $encloser, 'lineend')){
			$lineend_corrected = true;

			$corrected_content = self::correctLineend($corrected_content, $lineend);
		}

		if($encloser_corrected || $delimiter_corrected || $lineend_corrected){
			$corrected_content = $encloser . $corrected_content . $encloser;
		}

		return $corrected_content;
	}

	protected static function correctEscape(string $content): string{
		return str_replace('\\', '\\\\', $content);
	}

	protected static function correctEnclose(string $content, string $csv_enclose = '\''): string{
		return str_replace($csv_enclose, ('\\' . $csv_enclose), $content);
	}

	protected static function correctLineend(string $content, string $csv_lineend = self::LINEEND_WINDOWS): string{
		switch($csv_lineend){
			case self::LINEEND_WINDOWS:
				return str_replace(["\n", "\r"], "\\r\\n", $content);
			case self::LINEEND_UNIX:
			default:
				return str_replace(["\r\n", "\r"], "\\n", $content);
			case self::LINEEND_MAC:
				return str_replace(["\r\n", "\n"], "\\r", $content);
		}
	}

	/**
	 *
	 * @param string[] $fields
	 * @return void
	 */
	protected function checkWriteFieldNames(array $fields): void{
		if(!$this->options['csv_fieldnames'] ||
			$this->RefTable->current !== 1 ||
			!$this->fh
		){
			return;
		}
		$pos = 0;
		$content = '';

		foreach($fields as $field){
			'@phan-var string $field';
			$content .= $this->formatOutput(we_exim_Export::correctTagname($field, 'value', ++$pos));
		}

		$content .= self::lineends[$this->options['csv_lineend']] ?? self::lineends['windows']; //@phan-suppress-current-line PhanCoalescingNeverNull

		fwrite($this->fh, $content);
	}

	protected function fileComplete(): void{
		// to be overridden
	}

}
