<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: a9c27294dc17 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-01-15 19:30 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_XMLParser{
	/** @var string */
	var $parseError = '';

	/** @var array<int,array{name:string,value:string,attributes:array,next:int}> */
	var $Nodes = [];

	/** @var int */
	private $Handle = 0;

	/** @var array<string,int> */
	private $Mark;

	function parse(string &$data, string $encoding = 'UTF-8'): bool{
		if(empty($data)){
			return false;
		}
		//parser only supports ISO-8859-1, US-ASCII, UTF-8
		switch($encoding){
			case 'US-ASCII':
			case 'ISO-8859-1':
			case 'UTF-8':
				break;
			default:
				$encoding = 'UTF-8';
		}
		// Initialize the parser
		$parser = xml_parser_create($encoding);

		xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);

		//  XML_OPTION_SKIP_WHITE has to be set to 0
		// in php4 if the option is set to 1, all new line characters
		// will be removed from node content, even from a CDATA section
		xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 0);

		if(!xml_parse_into_struct($parser, $data, $this->Nodes)){
			$this->parseError = xml_get_current_line_number($parser) . ': ' . xml_Error_string(xml_get_error_code($parser) ?: 0) . ' - ' .
				xml_get_current_byte_index($parser) . '(' . substr($data, (int) (xml_get_current_byte_index($parser) - 10), 20) . ')';
			return false;
		}

		xml_parser_free($parser);
		//$this->normalize();
		return true;
	}

	function normalize(): void{
		$newNodes = [];

		$count = 0;
		$level = [];
		'@phan-var array<string,int> $level';
		$level_prefix = 'l';

		foreach($this->Nodes as $v){
			'@phan-var array<string,string> $v';
			switch($v['type']){
				case 'open':
				case 'complete':
					$new = [
						'name' => $v['tag']
					];

					if(isset($v['attributes'])){
						$new['attributes'] = $v['attributes'];
					}

					if(isset($v['value'])){
						$new['value'] = $v['value'];
					}

					$lvlPos = $level_prefix . $v['level'];
					if($count && isset($level[$lvlPos])){
						$newNodes[$level[$lvlPos]]['next'] = $count;
					}

					$level[$lvlPos] = $count;

					$newNodes[$count] = $new;
					$count++;
					break;
				case 'close':
					array_pop($level);
					break;
			}
		}

		$this->Nodes = $newNodes;
	}

	function next(): bool{
		if($this->Handle < (count($this->Nodes) - 1)){
			$this->Handle++;
			return true;
		}
		return false;
	}

	function nextSibling(): bool{
		if(isset($this->Nodes[$this->Handle]['next'])){
			$this->Handle = (int) $this->Nodes[$this->Handle]['next'];
			return true;
		}
		return false;
	}

	function seek(int $position): void{
		$this->Handle = $position;
	}

	function getNodeName(): string{
		return $this->Nodes[$this->Handle]['name'] ?? '';
	}

	function getNodeData(): string{
		return $this->Nodes[$this->Handle]['value'] ?? '';
	}

	function getNodeAttributes(): array{
		return (isset($this->Nodes[$this->Handle]['attributes'])) ?
			(array) $this->Nodes[$this->Handle]['attributes'] :
			[];
	}

	function addMark(string $name): void{
		$this->Mark[$name] = $this->Handle;
	}

	function gotoMark(string $name): bool{
		if(isset($this->Mark[$name])){
			$this->Handle = $this->Mark[$name];
			return true;
		}

		return false;
	}

	function deleteMark(string $name): bool{
		if(isset($this->Mark[$name])){
			unset($this->Mark[$name]);
			return true;
		}
		return false;
	}

	/**
	 *
	 * @param int $node_id
	 * @return int[]
	 */
	function getChildren(int $node_id): array{
		$children = [];
		$this->addMark('getChildren');
		$this->seek($node_id);

		$this->next();
		$children[] = $this->Handle;
		while($this->nextSibling()){
			$children[] = $this->Handle;
		}

		$this->gotoMark('getChildren');
		$this->deleteMark('getChildren');

		return $children;
	}

	function hasChildren(int $node_id): bool{
		$this->addMark('hasChildern');
		$this->seek($node_id);
		$next_id = 0;
		$next_sibling_id = 0;


		if($this->next()){
			$next_id = $this->Handle;
		}

		$this->seek($node_id);
		if($this->nextSibling()){
			$next_sibling_id = $this->Handle;
		}

		if($next_id != $next_sibling_id){
			return true;
		}

		$this->gotoMark('hasChildern');
		$this->deleteMark('hasChildern');
		return false;
	}

}
