<?php
declare(strict_types=1);

/**
 * Zend Framework
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @category   Zend
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

/**
 * @category   Zend
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class we_mail_MimeDecode{

	/**
	 * Explode MIME multipart string into seperate parts
	 *
	 * Parts consist of the header and the body of each MIME part.
	 *
	 * @param  string $body     raw body of message
	 * @param  string $boundary boundary as found in content-type
	 * @return string[] parts with content of each part, empty if no parts found
	 * @throws Exception
	 */
	public static function splitMime(string $body, string $boundary): array{
		// TODO: we're ignoring \r for now - is this function fast enough and is it safe to asume noone needs \r?
		$body = str_replace("\r", '', $body);

		$res = [];
		// find every mime part limiter and cut out the
		// string before it.
		// the part before the first boundary string is discarded:
		$p = strpos($body, '--' . $boundary . "\n");
		if($p === false){
			// no parts found!
			return [];
		}

		// position after first boundary line
		$start = $p + 3 + strlen($boundary);

		while(($p = strpos($body, '--' . $boundary . "\n", $start)) !== false){
			$res[] = substr($body, $start, $p - $start) ?: '';
			$start = $p + 3 + strlen($boundary);
		}

		// no more parts, find end boundary
		$p = strpos($body, '--' . $boundary . '--', $start);
		if($p === false){
			throw new we_exception_mail('Not a valid Mime Message: End Missing');
		}

		// the remaining part also needs to be parsed:
		$res[] = substr($body, $start, $p - $start) ?: '';

		return $res;
	}

	/**
	 * decodes a mime encoded String and returns a
	 * struct of parts with header and body
	 *
	 * @param  string $message  raw message content
	 * @param  string $boundary boundary as found in content-type
	 * @param  string $EOL      EOL string; defaults to
	 * @return array<int,array{header:array<string,string>,body:string}> parts as array('header' => array(name => value), 'body' => content), null if no parts found
	 * @throws Exception
	 */
	public static function splitMessageStruct(string $message, string $boundary, string $EOL = we_mail_mime::LINEEND): array{
		$parts = self::splitMime($message, $boundary);
		if(count($parts) <= 0){
			return [];
		}
		$result = [];
		foreach($parts as $part){
			'@phan-var string $part';
			$headers = [];
			$body = '';
			self::splitMessage($part, $headers, $body, $EOL);
			$result[] = [
				'header' => $headers,
				'body' => $body
			];
		}

		return $result;
	}

	/**
	 * split a message in header and body part, if no header or an
	 * invalid header is found $headers is empty
	 *
	 * The charset of the returned headers depend on your iconv settings.
	 *
	 * @param  string $message raw message with header and optional content
	 * @param  array  $headers output param, array with headers as array(name => value)
	 * @param  string $body    output param, content of message
	 * @param  string $EOL     EOL string; defaults to
	 * @return void
	 */
	public static function splitMessage(string $message, array &$headers, string &$body, string $EOL = we_mail_mime::LINEEND): void{
		// check for valid header at first line
		$firstline = (string) strtok($message, "\n");
		if(!preg_match('%^[^\s]+[^:]*:%', $firstline)){
			$headers = [];
			// TODO: we're ignoring \r for now - is this function fast enough and is it safe to asume noone needs \r?
			$body = str_replace(
				["\r",
					"\n"
				], ['',
				$EOL
				], $message
			);

			return;
		}

		// find an empty line between headers and body
		// default is set new line
		if(strpos($message, $EOL . $EOL)){
			[$headers, $body] = explode($EOL . $EOL, $message, 2);
			// next is the standard new line
		} else {
			if($EOL != "\r\n" && strpos($message, "\r\n\r\n")){
				[$headers, $body] = explode("\r\n\r\n", $message, 2);
				// next is the other "standard" new line
			} else {
				if($EOL != "\n" && strpos($message, "\n\n")){
					[$headers, $body] = explode("\n\n", $message, 2);
					// at last resort find anything that looks like a new line
				} else {
					[$headers, $body] = @preg_split("%([\r\n]+)\\1%U", $message, 2);
				}
			}
		}

		//FIXME: check if we can use mb_decode_mimeheader
		$headers = iconv_mime_decode_headers($headers, ICONV_MIME_DECODE_CONTINUE_ON_ERROR);

		if($headers === false){
			// an error occurs during the decoding
			return;
		}

		// normalize header names
		foreach($headers as $name => $header){
			$lower = strtolower($name);
			if($lower == $name){
				continue;
			}
			unset($headers[$name]);
			if(!isset($headers[$lower])){
				$headers[$lower] = $header;
				continue;
			}
			if(is_array($headers[$lower])){
				$headers[$lower][] = $header;
				continue;
			}
			$headers[$lower] = [$headers[$lower], $header];
		}
	}

	/**
	 * decode a quoted printable encoded string
	 *
	 * The charset of the returned string depends on your iconv settings.
	 *
	 * @param  string $string Encoded string
	 * @return string         Decoded string
	 */
	public static function decodeQuotedPrintable(string $string): string{
		return quoted_printable_decode($string);
	}

}
