<?php
declare(strict_types=1);

/**

 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

/**
 * Abstract for sending eMails through different
 * ways of transport
 *
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
abstract class we_mail_TransportAbstract{
	/**
	 * Mail body
	 * @var string
	 * @access public
	 */
	public $body = '';

	/**
	 * MIME boundary
	 * @var string
	 * @access public
	 */
	public $boundary = '';

	/**
	 * Mail header string
	 * @var string
	 * @access public
	 */
	public $header = '';

	/**
	 * Array of message headers
	 * @var array
	 * @access protected
	 */
	protected $headers = [];

	/**
	 * Message is a multipart message
	 * @var boolean
	 * @access protected
	 */
	protected $isMultipart = false;

	/**
	 * Mail object
	 * @var ?we_mail_znd
	 * @access protected
	 */
	protected $mail = null;

	/**
	 * Array of message parts
	 * @var array
	 * @access protected
	 */
	protected $parts = [];

	/**
	 * Recipients string
	 * @var string
	 * @access public
	 */
	public $recipients = '';

	/**
	 * EOL character string used by transport
	 * @var string
	 * @access public
	 */
	protected const EOL = "\r\n";

	/**
	 * Send an email independent from the used transport
	 *
	 * The requisite information for the email will be found in the following
	 * properties:
	 *
	 * - {@link $recipients} - list of recipients (string)
	 * - {@link $header} - message header
	 * - {@link $body} - message body
	 */
	abstract protected function sendMail(): void;

	/**
	 * Return all mail headers as an array
	 *
	 * If a boundary is given, a multipart header is generated with a
	 * Content-Type of either multipart/alternative or multipart/mixed depending
	 * on the mail parts present in the Mail object present.
	 *
	 * @param string $boundary
	 * @return array
	 */
	protected function getHeaders(string $boundary): array{
		if($boundary){
			// Build multipart mail
			$type = $this->mail->getType();
			if(!$type){
				if($this->mail->hasAttachments){
					$type = we_mail_mime::MULTIPART_MIXED;
				} elseif($this->mail->getBodyText() && $this->mail->getBodyHtml()){
					$type = we_mail_mime::MULTIPART_ALTERNATIVE;
				} else {
					$type = we_mail_mime::MULTIPART_MIXED;
				}
			}

			$this->headers['Content-Type'] = [$type . ';' . static::EOL . ' ' . 'boundary="' . $boundary . '"'];
			$this->boundary = $boundary;
		}

		$this->headers['MIME-Version'] = ['1.0'];

		return $this->headers;
	}

	/**
	 * Prepare header string for use in transport
	 *
	 * Prepares and generates {@link $header} based on the headers provided.
	 *
	 * @param array<string,string[]> $headers
	 * @access protected
	 * @return void
	 * @throws we_exception_mail if any header lines exceed 998
	 * characters
	 */
	protected function prepareHeaders(array $headers): void{
		if(!$this->mail){
			/**
			 * @see Mail_Transport_Exception
			 */
			throw new we_exception_mail('Missing Mail object in _mail property');
		}

		$this->header = '';

		foreach($headers as $header => $content){
			if(isset($content['append'])){
				unset($content['append']);
				$this->header .= $header . ': ' . implode(',' . static::EOL . ' ', $content) . static::EOL;
			} else {
				array_walk($content, function(&$item, $key, $prefix){//@phan-suppress-current-line PhanUnusedClosureParameter
						$item = $prefix . ': ' . $item;
					}, $header);
				$this->header .= implode(static::EOL, $content) . static::EOL;
			}
		}

		// Sanity check on headers -- should not be > 998 characters
		$sane = true;
		foreach(explode(static::EOL, $this->header) as $line){
			if(strlen(trim($line)) > 998){
				$sane = false;
				break;
			}
		}
		if(!$sane){
			/**
			 * @see Mail_Transport_Exception
			 */
			throw new we_exception_mail('At least one mail header line is too long');
		}
	}

	/**
	 * Generate MIME compliant message from the current configuration
	 *
	 * If both a text and HTML body are present, generates a
	 * multipart/alternative Mime_Part containing the headers and contents
	 * of each. Otherwise, uses whichever of the text or HTML parts present.
	 *
	 * The content part is then prepended to the list of Mime_Parts for
	 * this message.
	 *
	 * @return void
	 */
	protected function buildBody(){
		if(($text = $this->mail->getBodyText()) && ($html = $this->mail->getBodyHtml())){
			// Generate unique boundary for multipart/alternative
			$mime = new we_mail_mime();
			$boundaryLine = $mime->boundaryLine(static::EOL);
			$boundaryEnd = $mime->mimeEnd(static::EOL);

			$text->disposition = '';
			$html->disposition = '';

			$body = $boundaryLine .
				$text->getHeaders(static::EOL) .
				static::EOL .
				$text->getContent(static::EOL) .
				static::EOL .
				$boundaryLine .
				$html->getHeaders(static::EOL) .
				static::EOL .
				$html->getContent(static::EOL) .
				static::EOL .
				$boundaryEnd;

			$mp = new we_mail_mimePart($body);
			$mp->type = we_mail_mime::MULTIPART_ALTERNATIVE;
			$mp->boundary = $mime->boundary();

			$this->isMultipart = true;

			// Ensure first part contains text alternatives
			array_unshift($this->parts, $mp);

			// Get headers
			$this->headers = $this->mail->getHeaders();
			return;
		}

		// If not multipart, then get the body
		if(($body = $this->mail->getBodyHtml())){
			array_unshift($this->parts, $body);
		} elseif(($body = $this->mail->getBodyText())){
			array_unshift($this->parts, $body);
		}

		if(!$body){
			/**
			 * @see Mail_Transport_Exception
			 */
			throw new we_exception_mail('No body specified');
		}

		// Get headers
		$this->headers = $this->mail->getHeaders();
		$headers = $body->getHeadersArray(static::EOL);
		foreach($headers as $type => $name){
			// Headers in Mime_Part are kept as arrays with two elements, a
			// key and a value
			$this->headers[$type] = [$name];
		}
	}

	/**
	 * Send a mail using this transport
	 *
	 * @param  we_mail_znd $mail
	 * @access public
	 * @return void
	 * @throws we_exception_mail if mail is empty
	 */
	public function send(we_mail_znd $mail): void{
		$this->isMultipart = false;
		$this->mail = $mail;
		$this->parts = $mail->getParts();
		$mime = $mail->getMime();

		// Build body content
		$this->buildBody();

		// Determine number of parts and boundary
		$count = count($this->parts);
		if($count < 1){
			/**
			 * @see Mail_Transport_Exception
			 */
			throw new we_exception_mail('Empty mail cannot be sent');
		}

		$boundary = '';
		if($count > 1){
			// Multipart message; create new MIME object and boundary
			$mime = new we_mail_mime($this->mail->getMimeBoundary());
			$boundary = $mime->boundary();
		} elseif($this->isMultipart){
			// multipart/alternative -- grab boundary
			$boundary = $this->parts[0]->boundary;
		}

		// Determine recipients, and prepare headers
		$this->recipients = implode(',', $mail->getRecipients());
		$this->prepareHeaders($this->getHeaders($boundary));

		// Create message body
		// This is done so that the same Mail object can be used in
		// multiple transports
		$message = new we_mail_mimeMessage();
		$message->setParts($this->parts);
		$message->setMime($mime);
		$this->body = $message->generateMessage(static::EOL);

		// Send to transport!
		$this->sendMail();
	}

}
